package com.smppw.modaq.infrastructure.util;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.modaq.common.enums.ReportType;
import net.sf.sevenzipjbinding.ExtractOperationResult;
import net.sf.sevenzipjbinding.IInArchive;
import net.sf.sevenzipjbinding.SevenZip;
import net.sf.sevenzipjbinding.SevenZipException;
import net.sf.sevenzipjbinding.impl.RandomAccessFileInStream;
import net.sf.sevenzipjbinding.simple.ISimpleInArchive;
import net.sf.sevenzipjbinding.simple.ISimpleInArchiveItem;
import org.apache.commons.compress.archivers.ArchiveEntry;
import org.apache.commons.compress.archivers.ArchiveException;
import org.apache.commons.compress.archivers.ArchiveInputStream;
import org.apache.commons.compress.archivers.ArchiveStreamFactory;

import java.io.*;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Pattern;

public class ExcelUtil {
    // 候选编码列表（按常见顺序排列）
    private static final List<String> CANDIDATE_ENCODINGS = Arrays.asList(
            "GBK",      // 中文环境常用
            "UTF-8",   // 标准编码
            "GB2312",  // 旧版中文
            "ISO-8859-1" // 默认回退
    );

    private static final Pattern ILLEGAL_CHARS_PATTERN =
            Pattern.compile("[\\\\:*?\"<>|\\x00-\\x1F]"); // 包含控制字符

    // Unicode 中文字符区块定义
    private static final int[][] CJK_BLOCKS = {
            {0x4E00, 0x9FFF},    // CJK Unified Ideographs (基本汉字)
            {0x3400, 0x4DBF},    // CJK Extension A (生僻字)
            {0x20000, 0x2A6DF},  // CJK Extension B (极生僻字)
            {0x2A700, 0x2B73F},  // CJK Extension C
            {0x3000, 0x303F}     // CJK标点符号
    };

    // 常见高频汉字范围（覆盖约99%日常用字）
    private static final int[][] COMMON_CHINESE_BLOCKS = {
            {0x4E00, 0x9FA5},    // 通用规范汉字表（8105字）
            {0x3000, 0x303F}     // 常用标点
    };

    public static boolean isExcel(String fileName) {
        return StrUtil.isNotBlank(fileName) && (fileName.endsWith("xls") || fileName.endsWith("xlsx") || fileName.endsWith("XLS") || fileName.endsWith("XLSX"));
    }

    public static boolean isPdf(String fileName) {
        return StrUtil.isNotBlank(fileName) && (fileName.endsWith("pdf") || fileName.endsWith("PDF"));
    }

    public static boolean isZip(String fileName) {
        return StrUtil.isNotBlank(fileName) && (fileName.endsWith("zip") || fileName.endsWith("ZIP"));
    }

    public static boolean isHTML(String fileName) {
        return StrUtil.isNotBlank(fileName) && fileName.endsWith("html");
    }

    public static boolean isRAR(String fileName) {
        return StrUtil.isNotBlank(fileName) && (fileName.endsWith("rar") || fileName.endsWith("RAR"));
    }

    public static List<String> extractCompressedFiles(String zipFilePath, String destFilePath) throws IOException, ArchiveException {
        File destFile = FileUtil.file(destFilePath);
        if (!destFile.exists()) {
            Files.createDirectories(destFile.toPath());
        }

        String encoding = detectEncoding(zipFilePath);
        if (encoding == null) {
            encoding = "GBK";
        }

        return ZipUtil.decompressZip(zipFilePath, destFilePath, 2, encoding);
    }

    public static List<String> extractRar5(String rarFilePath, String outputDir) throws Exception {
        // 初始化 SevenZipJBinding 本地库
        SevenZip.initSevenZipFromPlatformJAR();

        RandomAccessFile randomAccessFile = null;
        IInArchive inArchive = null;

        List<String> resultList = ListUtil.list(false);
        try {
            // 打开 RAR 文件
            randomAccessFile = new RandomAccessFile(rarFilePath, "r");
            inArchive = SevenZip.openInArchive(null, new RandomAccessFileInStream(randomAccessFile));

            // 获取压缩包中的文件列表
            ISimpleInArchive simpleInArchive = inArchive.getSimpleInterface();
            for (ISimpleInArchiveItem item : simpleInArchive.getArchiveItems()) {
                if (!item.isFolder()) {
                    resultList.add(extractItem(item, outputDir));
                }
            }
        } finally {
            // 释放资源
            if (inArchive != null) {
                inArchive.close();
            }
            if (randomAccessFile != null) {
                randomAccessFile.close();
            }
        }
        return resultList;
    }

    private static String extractItem(ISimpleInArchiveItem item, String outputDir) throws SevenZipException {
        String filePath = outputDir + File.separator + item.getPath();
        File outputFile = FileUtil.file(filePath);

        // 创建父目录
        File parentDir = outputFile.getParentFile();
        if (!parentDir.exists() && !parentDir.mkdirs()) {
            throw new SevenZipException("无法创建目录: " + parentDir.getAbsolutePath());
        }

        // 提取文件内容
        try (FileOutputStream fos = new FileOutputStream(outputFile)) {
            ExtractOperationResult result = item.extractSlow(data -> {
                try {
                    fos.write(data);
                    return data.length; // 返回写入的字节数
                } catch (IOException e) {
                    throw new SevenZipException("写入文件失败", e);
                }
            });

            if (result != ExtractOperationResult.OK) {
                throw new SevenZipException("解压失败: " + result);
            }
        } catch (IOException e) {
            throw new SevenZipException("文件操作失败", e);
        }
        return outputFile.getAbsolutePath();
    }

    // 检测压缩包编码
    private static String detectEncoding(String zipPath) {
        for (String encoding : CANDIDATE_ENCODINGS) {
            try (BufferedInputStream fis = new BufferedInputStream(new FileInputStream(zipPath));
                 ArchiveInputStream<? extends ArchiveEntry> ais = new ArchiveStreamFactory()
                         .createArchiveInputStream(ArchiveStreamFactory.detect(fis), fis, encoding)) {

                ArchiveEntry entry = ais.getNextEntry();
                if (entry == null) continue; // 空压缩包

                String fileName = entry.getName();
                if (!isLikelyGarbled(fileName, encoding)) {
                    return encoding; // 找到有效编码
                }
            } catch (Exception e) {
                // 编码不支持或文件错误，继续尝试下一个
            }
        }
        return null;
    }

    public static boolean isLikelyGarbled(String fileName, String encoding) {
        // 基础检查：非法字符、替换符、连续问号
        if (ILLEGAL_CHARS_PATTERN.matcher(fileName).find() ||
                fileName.contains("�") ||
                fileName.matches(".*\\?{2,}.*")) {
            return true;
        }

        // 编码一致性检查（假设系统使用 UTF-8）
        if (!isEncodingConsistent(fileName, Charset.forName(encoding))) {
            return true;
        }

        // 中文字符乱码检测
        return hasLowMeaningfulness(fileName) || isLikelyGarbledWithRareChars(fileName);
    }

    private static boolean isEncodingConsistent(String text, Charset expectedCharset) {
        // 将文本按预期编码转换为字节，再解码验证一致性
        byte[] bytes = text.getBytes(expectedCharset);
        String redecoded = new String(bytes, expectedCharset);
        return text.equals(redecoded);
    }

    public static boolean isLikelyGarbledWithRareChars(String text) {
        int totalChars = text.length();
        if (totalChars == 0) return false;

        int commonCount = 0;
        int rareCJKCount = 0;

        for (char c : text.toCharArray()) {
            // 判断是否属于任何CJK区块
            boolean isCJK = isInUnicodeBlocks(c, CJK_BLOCKS);
            // 判断是否属于高频汉字区
            boolean isCommon = isInUnicodeBlocks(c, COMMON_CHINESE_BLOCKS);

            if (isCJK && !isCommon) {
                rareCJKCount++; // 统计生僻CJK字符
            } else if (isCommon) {
                commonCount++;
            }
        }

        // 规则1：生僻CJK占比超过50%且总CJK字符占比高
        boolean rule1 = (rareCJKCount > 0) &&
                (rareCJKCount * 2 > totalChars) &&
                ((commonCount + rareCJKCount) * 1.0 / totalChars > 0.7);

        // 规则2：高频字占比极低（<20%）但CJK总占比高（编码错误特征）
        boolean rule2 = (commonCount * 1.0 / totalChars < 0.2) &&
                ((commonCount + rareCJKCount) * 1.0 / totalChars > 0.6);

        return rule1 || rule2;
    }

    // 辅助方法：判断字符是否在指定Unicode区块内
    private static boolean isInUnicodeBlocks(char c, int[][] blocks) {
        for (int[] block : blocks) {
            if ((int) c >= block[0] && (int) c <= block[1]) {
                return true;
            }
        }
        return false;
    }

    // 上下文合理性检测
    private static boolean hasLowMeaningfulness(String text) {
        // 假设正常文本应包含常见停用词（的、是、在等）
        List<String> commonWords = ListUtil.list(false);
        commonWords.add("基金");
        commonWords.addAll(ListUtil.toList(ReportType.ANNUALLY.getPatterns()));
        commonWords.addAll(ListUtil.toList(ReportType.QUARTERLY.getPatterns()));
        commonWords.addAll(ListUtil.toList(ReportType.MONTHLY.getPatterns()));
        commonWords.addAll(ListUtil.toList(ReportType.LETTER.getPatterns()));
        commonWords.addAll(ListUtil.toList(ReportType.WEEKLY.getPatterns()));
        commonWords.addAll(ListUtil.toList(ReportType.OTHER.getPatterns()));
        for (String word : commonWords) {
            if (text.contains(word)) return false;
        }
        return true;
    }

    public static void main(String[] args) throws Exception {
        String zipFilePath = "D:\\home\\wwwroot\\mo_report_file\\wangzaijun@simuwang.com\\20250321\\20250321143709排排网确认单.rar";
        String destFilePath = "D:\\home\\wwwroot\\mo_report_file\\wangzaijun@simuwang.com\\20250321";
        List<String> strings = extractRar5(zipFilePath, destFilePath);
        for (String string : strings) {
            System.out.println(string);
        }
//        List<String> fileList = extractCompressedFiles(zipFilePath, destFilePath);
//        for (String s : fileList) {
//            System.out.println(s);
//        }
    }
}
