package com.smppw.modaq.infrastructure.util;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.io.FileUtil;
import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipFile;
import org.apache.commons.io.IOUtils;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Enumeration;
import java.util.List;

public class ZipUtil {
    /**
     * 递归解压 ZIP 文件（含嵌套深度限制）
     *
     * @param zipFile   输入的 ZIP 文件
     * @param outputDir 解压目标根目录
     * @param maxDepth  最大嵌套深度（例如 3 表示允许 parent/nest1/nest2.zip）
     * @return 所有解压后的文件路径（格式：parent.zip/nest1/file.txt）
     */
    public static List<String> decompressZip(String zipFile, String outputDir, int maxDepth, String encoding) throws IOException {
        return decompressZip(FileUtil.file(zipFile), FileUtil.file(outputDir), maxDepth, encoding);
    }

    /**
     * 递归解压 ZIP 文件（含嵌套深度限制）
     *
     * @param zipFile   输入的 ZIP 文件
     * @param outputDir 解压目标根目录
     * @param maxDepth  最大嵌套深度（例如 3 表示允许 parent.zip/nest1.zip/nest2.zip）
     * @return 所有解压后的文件路径（格式：parent.zip/nest1.zip/file.txt）
     */
    public static List<String> decompressZip(File zipFile, File outputDir, int maxDepth, String encoding) throws IOException {
        if (maxDepth < 0) {
            throw new IllegalArgumentException("最大嵌套深度不能小于 0");
        }
        List<String> decompressedFiles = ListUtil.list(false);
        decompressZipRecursive(zipFile, outputDir, "", 0, maxDepth, encoding, decompressedFiles);
        return decompressedFiles;
    }

    /**
     * 递归解压核心逻辑
     */
    private static void decompressZipRecursive(
            File currentZip,
            File rootOutputDir,
            String nestedPath,
            int currentDepth,
            int maxDepth,
            String encoding,
            List<String> decompressedFiles) throws IOException {

        // 1. 超过最大深度时停止处理嵌套 ZIP
        if (currentDepth > maxDepth) {
            return;
        }

        // 2. 创建当前 ZIP 的解压目录（跟压缩包目录已经处理过，就不要追加到文件目录中了）
        String currentZipName = FileUtil.mainName(currentZip);
        String currentNestedPath = nestedPath.isEmpty()
                ? ""
                : nestedPath + File.separator + currentZipName;
        File currentOutputDir = new File(rootOutputDir, currentNestedPath);
        FileUtil.mkdir(currentOutputDir);

        // 3. 解压当前 ZIP，支持最多10个分卷的解压
        try (ZipFile zip = ZipFile.builder().setFile(currentZip).setCharset(encoding).setMaxNumberOfDisks(10).get()) {
            Enumeration<ZipArchiveEntry> entries = zip.getEntries();

            while (entries.hasMoreElements()) {
                ZipArchiveEntry entry = entries.nextElement();
                String name = entry.getName();
                if (name.startsWith("__MACOSX/")) {
                    continue;
                }

                Path entryPath = Paths.get(currentOutputDir.getAbsolutePath(), name);

                // 处理目录
                if (entry.isDirectory()) {
                    Files.createDirectories(entryPath);
                    continue;
                }

                // 写入文件
                Files.createDirectories(entryPath.getParent());
                try (InputStream is = zip.getInputStream(entry);
                     OutputStream os = new FileOutputStream(entryPath.toFile())) {
                    IOUtils.copy(is, os);
                }

                // 4. 递归处理嵌套 ZIP（深度+1）
                if (isZipFile(name) && currentDepth < maxDepth) {
                    File nestedZipFile = entryPath.toFile();
                    decompressZipRecursive(
                            nestedZipFile,
                            rootOutputDir,
                            currentNestedPath,
                            currentDepth + 1,  // 深度递增
                            maxDepth,
                            encoding,
                            decompressedFiles
                    );
                    Files.delete(nestedZipFile.toPath());
                } else {
                    // 记录路径
                    decompressedFiles.add(entryPath.toString());
                }
            }
        }
    }

    /**
     * 判断文件是否为 ZIP 格式
     */
    private static boolean isZipFile(String filepath) {
        return filepath.toLowerCase().endsWith(".zip");
    }

    // 使用示例
    public static void main(String[] args) throws Exception {
//        List<String> files = decompressZip(
//                new File("C:\\Users\\Administrator\\Desktop\\上海熙盛明诚私募基金管理有限公司-信披报告.zip"),
//                new File("C:\\Users\\Administrator\\Desktop"),
//                2,
//                "UTF-8"
//        );
//        System.out.println("解压后的文件路径:");
//        files.forEach(System.out::println);

        String currentZip = "D:\\Documents\\新报告解析\\确认单\\20250514_份额及交易确认函_上海量魁私募基金管理有限公司_深圳市前海排排网基金销售有限责任公司_TA确认数据.zip";
        List<String> files = decompressZip(currentZip, "D:\\Documents\\新报告解析\\确认单\\", 2, "utf-8");
        System.out.println("解压后的文件路径:");
        files.forEach(System.out::println);
    }
}
